@@ -1,7 +1,7 @@
 /*
  * VncConnection.cpp - implementation of VncConnection class
  *
- * Copyright (c) 2008-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2008-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -176,11 +176,7 @@ VncConnection::VncConnection( QObject* parent ) :
 	m_globalMutex(),
 	m_eventQueueMutex(),
 	m_updateIntervalSleeper(),
-	m_framebufferUpdateInterval( 0 ),
-	m_image(),
-	m_scaledScreen(),
-	m_scaledSize(),
-	m_imgLock()
+	m_framebufferUpdateInterval( 0 )
 {
 	if( VeyonCore::config().useCustomVncConnectionSettings() )
 	{
@@ -201,8 +197,6 @@ VncConnection::VncConnection( QObject* parent ) :
 
 VncConnection::~VncConnection()
 {
-	stop();
-
 	if( isRunning() )
 	{
 		vWarning() << "Waiting for VNC connection thread to finish.";
@@ -255,7 +249,7 @@ void VncConnection::stop()
 {
 	setClientData( VncConnectionTag, nullptr );
 
-	m_scaledScreen = {};
+	m_scaledFramebuffer = {};
 
 	setControlFlag( ControlFlag::TerminateThread, true );
 
@@ -268,12 +262,12 @@ void VncConnection::stopAndDeleteLater()
 {
 	if( isRunning() )
 	{
-		connect( this, &VncConnection::finished, this, &VncConnection::deleteLater );
+		setControlFlag( ControlFlag::DeleteAfterFinished, true );
 		stop();
 	}
 	else
 	{
-		deleteLater();
+		deleteLaterInMainThread();
 	}
 }
 
@@ -325,6 +319,20 @@ void VncConnection::setPort( int port )
 
 
 
+void VncConnection::setUseRemoteCursor( bool enabled )
+{
+	m_useRemoteCursor = enabled;
+
+	if( m_client )
+	{
+		m_client->appData.useRemoteCursor = enabled ? TRUE : FALSE;
+
+		enqueueEvent(new VncUpdateFormatAndEncodingsEvent);
+	}
+}
+
+
+
 void VncConnection::setServerReachable()
 {
 	setControlFlag( ControlFlag::ServerReachable, true );
@@ -339,16 +347,16 @@ void VncConnection::setScaledSize( QSize s )
 	if( m_scaledSize != s )
 	{
 		m_scaledSize = s;
-		setControlFlag( ControlFlag::ScaledScreenNeedsUpdate, true );
+		setControlFlag( ControlFlag::ScaledFramebufferNeedsUpdate, true );
 	}
 }
 
 
 
-QImage VncConnection::scaledScreen()
+QImage VncConnection::scaledFramebuffer()
 {
-	rescaleScreen();
-	return m_scaledScreen;
+	rescaleFramebuffer();
+	return m_scaledFramebuffer;
 }
 
 
@@ -356,19 +364,26 @@ QImage VncConnection::scaledScreen()
 void VncConnection::setFramebufferUpdateInterval( int interval )
 {
 	m_framebufferUpdateInterval = interval;
+
+	if (m_framebufferUpdateInterval <= 0)
+	{
+		setControlFlag(ControlFlag::TriggerFramebufferUpdate, true);
+	}
+
+	m_updateIntervalSleeper.wakeAll();
 }
 
 
 
-void VncConnection::rescaleScreen()
+void VncConnection::rescaleFramebuffer()
 {
 	if( hasValidFramebuffer() == false || m_scaledSize.isNull() )
 	{
-		m_scaledScreen = {};
+		m_scaledFramebuffer = {};
 		return;
 	}
 
-	if( isControlFlagSet( ControlFlag::ScaledScreenNeedsUpdate ) == false )
+	if( isControlFlagSet( ControlFlag::ScaledFramebufferNeedsUpdate ) == false )
 	{
 		return;
 	}
@@ -380,9 +395,9 @@ void VncConnection::rescaleScreen()
 		return;
 	}
 
-	m_scaledScreen = m_image.scaled( m_scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
+	m_scaledFramebuffer = m_image.scaled( m_scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
 
-	setControlFlag( ControlFlag::ScaledScreenNeedsUpdate, false );
+	setControlFlag( ControlFlag::ScaledFramebufferNeedsUpdate, false );
 }
 
 
@@ -419,6 +434,11 @@ void VncConnection::run()
 		handleConnection();
 		closeConnection();
 	}
+
+	if( isControlFlagSet( ControlFlag::DeleteAfterFinished ) )
+	{
+		deleteLaterInMainThread();
+	}
 }
 
 
@@ -467,13 +487,23 @@ void VncConnection::establishConnection()
 
 		setControlFlag( ControlFlag::ServerReachable, false );
 
-		if( rfbInitClient( m_client, nullptr, nullptr ) &&
-			isControlFlagSet( ControlFlag::TerminateThread ) == false )
+		const auto clientInitialized = rfbInitClient( m_client, nullptr, nullptr );
+		if( clientInitialized == FALSE )
+		{
+			// rfbInitClient() calls rfbClientCleanup() when failed
+			m_client = nullptr;
+		}
+
+		// do not continue/sleep when already requested to stop
+		if( isControlFlagSet( ControlFlag::TerminateThread ) )
+		{
+			return;
+		}
+
+		if( clientInitialized )
 		{
 			m_framebufferUpdateWatchdog.restart();
 
-			Q_EMIT connectionEstablished();
-
 			VeyonCore::platform().networkFunctions().
 					configureSocketKeepalive( static_cast<PlatformNetworkFunctions::Socket>( m_client->sock ), true,
 											  m_socketKeepaliveIdleTime, m_socketKeepaliveInterval, m_socketKeepaliveCount );
@@ -482,19 +512,11 @@ void VncConnection::establishConnection()
 		}
 		else
 		{
-			// rfbInitClient() calls rfbClientCleanup() when failed
-			m_client = nullptr;
-
-			// do not sleep when already requested to stop
-			if( isControlFlagSet( ControlFlag::TerminateThread ) )
-			{
-				break;
-			}
-
 			// guess reason why connection failed
 			if( isControlFlagSet( ControlFlag::ServerReachable ) == false )
 			{
-				if( VeyonCore::platform().networkFunctions().ping( m_host ) == false )
+				if( isControlFlagSet( ControlFlag::SkipHostPing ) ||
+					VeyonCore::platform().networkFunctions().ping( m_host ) == false )
 				{
 					setState( State::HostOffline );
 				}
@@ -517,12 +539,12 @@ void VncConnection::establishConnection()
 			sleeperMutex.lock();
 			if( m_framebufferUpdateInterval > 0 )
 			{
-				m_updateIntervalSleeper.wait( &sleeperMutex, QDeadlineTimer( m_framebufferUpdateInterval ) );
+				m_updateIntervalSleeper.wait( &sleeperMutex, m_framebufferUpdateInterval );
 			}
 			else
 			{
 				// default: retry every second
-				m_updateIntervalSleeper.wait( &sleeperMutex, QDeadlineTimer( m_connectionRetryInterval ) );
+				m_updateIntervalSleeper.wait( &sleeperMutex, m_connectionRetryInterval );
 			}
 			sleeperMutex.unlock();
 		}
@@ -542,7 +564,8 @@ void VncConnection::handleConnection()
 	{
 		loopTimer.start();
 
-		const int i = WaitForMessage( m_client, m_messageWaitTimeout );
+		const int i = WaitForMessage(m_client, m_framebufferUpdateInterval > 0 ?
+													m_messageWaitTimeout * 100 : m_messageWaitTimeout);
 		if( isControlFlagSet( ControlFlag::TerminateThread ) || i < 0 )
 		{
 			break;
@@ -560,28 +583,32 @@ void VncConnection::handleConnection()
 				break;
 			}
 		}
-
-		sendEvents();
+		else if (m_framebufferUpdateWatchdog.elapsed() >=
+				 qMax<qint64>(2*m_framebufferUpdateInterval, m_framebufferUpdateWatchdogTimeout))
+		{
+			SendFramebufferUpdateRequest(m_client, 0, 0, m_client->width, m_client->height, false);
+			m_framebufferUpdateWatchdog.restart();
+		}
+		else if (m_framebufferUpdateInterval > 0 && m_framebufferUpdateWatchdog.elapsed() > m_framebufferUpdateInterval)
+		{
+			SendIncrementalFramebufferUpdateRequest(m_client);
+			m_framebufferUpdateWatchdog.restart();
+		}
+		else if (isControlFlagSet(ControlFlag::TriggerFramebufferUpdate))
+		{
+			setControlFlag(ControlFlag::TriggerFramebufferUpdate, false);
+			SendIncrementalFramebufferUpdateRequest(m_client);
+		}
 
 		const auto remainingUpdateInterval = m_framebufferUpdateInterval - loopTimer.elapsed();
 
-		if( m_framebufferState == FramebufferState::Initialized ||
-			m_framebufferUpdateWatchdog.elapsed() >= qMax<qint64>( 2*m_framebufferUpdateInterval, m_framebufferUpdateWatchdogTimeout ) )
-		{
-			SendFramebufferUpdateRequest( m_client, 0, 0, m_client->width, m_client->height, false );
-
-			const auto remainingFastUpdateInterval = m_fastFramebufferUpdateInterval - loopTimer.elapsed();
-
-			sleeperMutex.lock();
-			m_updateIntervalSleeper.wait( &sleeperMutex, QDeadlineTimer( remainingFastUpdateInterval ) );
-			sleeperMutex.unlock();
-		}
-		else if( m_framebufferState == FramebufferState::Valid &&
-			remainingUpdateInterval > 0 &&
-			isControlFlagSet( ControlFlag::TerminateThread ) == false )
+		// compat with Veyon Server < 4.7
+		if (remainingUpdateInterval > 0 &&
+			isControlFlagSet(ControlFlag::RequiresManualUpdateRateControl) &&
+			isControlFlagSet(ControlFlag::TerminateThread) == false)
 		{
 			sleeperMutex.lock();
-			m_updateIntervalSleeper.wait( &sleeperMutex, QDeadlineTimer( remainingUpdateInterval ) );
+			m_updateIntervalSleeper.wait( &sleeperMutex, remainingUpdateInterval );
 			sleeperMutex.unlock();
 		}
 
@@ -665,7 +692,7 @@ bool VncConnection::initFrameBuffer( rfbClient* client )
 	client->format.blueMax = 0xff;
 
 	client->appData.encodingsString = "zrle ultra copyrect hextile zlib corre rre raw";
-	client->appData.useRemoteCursor = false;
+	client->appData.useRemoteCursor = m_useRemoteCursor ? TRUE : FALSE;
 	client->appData.compressLevel = 0;
 	client->appData.useBGR233 = false;
 	client->appData.qualityLevel = 9;
@@ -677,9 +704,6 @@ bool VncConnection::initFrameBuffer( rfbClient* client )
 		// make sure to use lossless raw encoding
 		client->appData.encodingsString = "raw";
 		break;
-	case Quality::RemoteControl:
-		client->appData.useRemoteCursor = true;
-		break;
 	case Quality::Thumbnail:
 		client->appData.compressLevel = 9;
 		client->appData.qualityLevel = 5;
@@ -703,7 +727,7 @@ void VncConnection::finishFrameBufferUpdate()
 	m_framebufferUpdateWatchdog.restart();
 
 	m_framebufferState = FramebufferState::Valid;
-	setControlFlag( ControlFlag::ScaledScreenNeedsUpdate, true );
+	setControlFlag( ControlFlag::ScaledFramebufferNeedsUpdate, true );
 
 	Q_EMIT framebufferUpdateComplete();
 }
@@ -737,7 +761,14 @@ void VncConnection::sendEvents()
 
 
 
-void VncConnection::enqueueEvent( VncEvent* event, bool wake )
+void VncConnection::deleteLaterInMainThread()
+{
+	QTimer::singleShot( 0, VeyonCore::instance(), [this]() { delete this; } );
+}
+
+
+
+void VncConnection::enqueueEvent(VncEvent* event)
 {
 	if( state() != State::Connected )
 	{
@@ -748,10 +779,7 @@ void VncConnection::enqueueEvent( VncEvent* event, bool wake )
 	m_eventQueue.enqueue( event );
 	m_eventQueueMutex.unlock();
 
-	if( wake )
-	{
-		m_updateIntervalSleeper.wakeAll();
-	}
+	m_updateIntervalSleeper.wakeAll();
 }
 
 
@@ -766,21 +794,21 @@ bool VncConnection::isEventQueueEmpty()
 
 void VncConnection::mouseEvent( int x, int y, int buttonMask )
 {
-	enqueueEvent( new VncPointerEvent( x, y, buttonMask ), true );
+	enqueueEvent(new VncPointerEvent(x, y, buttonMask));
 }
 
 
 
 void VncConnection::keyEvent( unsigned int key, bool pressed )
 {
-	enqueueEvent( new VncKeyEvent( key, pressed ), true );
+	enqueueEvent(new VncKeyEvent(key, pressed));
 }
 
 
 
 void VncConnection::clientCut( const QString& text )
 {
-	enqueueEvent( new VncClientCutEvent( text ), true );
+	enqueueEvent(new VncClientCutEvent(text));
 }
 
 
