@@ -1,7 +1,7 @@
 /*
  * DocumentationFigureCreator.cpp - helper for creating documentation figures
  *
- * Copyright (c) 2019-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2019-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -55,19 +55,9 @@
 
 #ifdef VEYON_DEBUG
 
-DocumentationFigureCreator::DocumentationFigureCreator() :
-	QObject(),
-	m_master( nullptr ),
-	m_eventLoop( this )
-{
-	m_master = new VeyonMaster;
-}
-
-
-
 void DocumentationFigureCreator::run()
 {
-	auto mainWindow = m_master->mainWindow();
+	auto mainWindow = m_master.mainWindow();
 
 	mainWindow->move( 0, 0 );
 	mainWindow->resize( 3000, 1000 );
@@ -77,6 +67,7 @@ void DocumentationFigureCreator::run()
 
 	createFeatureFigures();
 	createContextMenuFigure();
+	createDemoMenuFigure();
 	createLogonDialogFigure();
 	createLocationDialogFigure();
 	createScreenshotManagementPanelFigure();
@@ -84,8 +75,8 @@ void DocumentationFigureCreator::run()
 	createTextMessageDialogFigure();
 	createOpenWebsiteDialogFigure();
 	createWebsiteMenuFigure();
-	createRunProgramDialogFigure();
-	createProgramMenuFigure();
+	createStartAppDialogFigure();
+	createAppMenuFigure();
 	createRemoteAccessHostDialogFigure();
 	createRemoteAccessWindowFigure();
 	createPowerDownOptionsFigure();
@@ -103,18 +94,18 @@ void DocumentationFigureCreator::createFeatureFigures()
 	int x = -1;
 	int w = 0;
 
-	auto toolbar = m_master->mainWindow()->findChild<MainToolBar *>();
+	auto toolbar = m_master.mainWindow()->findChild<MainToolBar *>();
 
 	const QStringList separatedPluginFeatures( { QStringLiteral("a54ee018-42bf-4569-90c7-0d8470125ccf"),
 												 QStringLiteral("80580500-2e59-4297-9e35-e53959b028cd")
 											   } );
 
-	const auto& features = m_master->features();
+	const auto& features = m_master.features();
 
 	for( const auto& feature : features )
 	{
 		auto btn = toolbar->findChild<ToolButton *>( feature.name() );
-		const auto pluginUid = m_master->featureManager().pluginUid( feature );
+		const auto pluginUid = VeyonCore::featureManager().pluginUid( feature.uid() );
 
 		if( previousPluginUid.isNull() )
 		{
@@ -162,7 +153,7 @@ void DocumentationFigureCreator::createFeatureFigures()
 
 void DocumentationFigureCreator::createContextMenuFigure()
 {
-	auto view = m_master->mainWindow()->findChild<ComputerMonitoringWidget *>();
+	auto view = m_master.mainWindow()->findChild<ComputerMonitoringWidget *>();
 	auto menu = view->findChild<QMenu *>();
 
 	connect( menu, &QMenu::aboutToShow, this, [menu]() {
@@ -177,7 +168,7 @@ void DocumentationFigureCreator::createContextMenuFigure()
 
 void DocumentationFigureCreator::createLogonDialogFigure()
 {
-	PasswordDialog dialog( m_master->mainWindow() );
+	PasswordDialog dialog( m_master.mainWindow() );
 	dialog.show();
 	dialog.findChild<QLineEdit *>( QStringLiteral("password") )->setText( QStringLiteral( "TeacherPassword") );
 	dialog.findChild<QLineEdit *>( QStringLiteral("password") )->cursorForward( false );
@@ -201,7 +192,7 @@ void DocumentationFigureCreator::createLocationDialogFigure()
 	}
 
 	QStringListModel locationsModel( locations, this );
-	LocationDialog dialog( &locationsModel, m_master->mainWindow() );
+	LocationDialog dialog( &locationsModel, m_master.mainWindow() );
 
 	grabDialog( &dialog, QSize( 300, 200 ), QStringLiteral("LocationDialog.png") );
 
@@ -212,7 +203,7 @@ void DocumentationFigureCreator::createLocationDialogFigure()
 
 void DocumentationFigureCreator::createScreenshotManagementPanelFigure()
 {
-	auto window = m_master->mainWindow();
+	auto window = m_master.mainWindow();
 	auto panel = window->findChild<ScreenshotManagementPanel *>();
 	auto panelButton = window->findChild<QToolButton *>( QStringLiteral("screenshotManagementPanelButton") );
 	auto list = panel->findChild<QListView *>();
@@ -272,29 +263,16 @@ void DocumentationFigureCreator::createScreenshotManagementPanelFigure()
 
 
 
+void DocumentationFigureCreator::createDemoMenuFigure()
+{
+	grabMenu( m_master.mainWindow(), QStringLiteral("Demo"), QStringLiteral("DemoMenu.png") );
+}
+
+
+
 void DocumentationFigureCreator::createPowerDownOptionsFigure()
 {
-	auto toolbar = m_master->mainWindow()->findChild<MainToolBar *>();
-	auto powerDownButton = toolbar->findChild<ToolButton *>( QStringLiteral("PowerDown") );
-
-	scheduleUiOperation( [this, powerDownButton]() {
-		scheduleUiOperation( [this, powerDownButton]() {
-			auto menu = powerDownButton->menu();
-
-			grabWindow( m_master->mainWindow(), powerDownButton->mapTo( m_master->mainWindow(), QPoint( 0, 0 ) ),
-						QSize( qMax( powerDownButton->width(), menu->width() ),
-							   powerDownButton->height() + menu->height() ),
-						QStringLiteral("PowerDownOptions.png") );
-			menu->close();
-		} );
-		auto menu = powerDownButton->menu();
-		menu->close();
-
-		powerDownButton->click();
-
-	} );
-
-	powerDownButton->showMenu();
+	grabMenu( m_master.mainWindow(), QStringLiteral("PowerDown"), QStringLiteral("PowerDownOptions.png") );
 }
 
 
@@ -309,7 +287,7 @@ void DocumentationFigureCreator::createPowerDownTimeInputDialogFigure()
 		grabDialog( dialog, {}, QStringLiteral("PowerDownTimeInputDialog.png") );
 	});
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "352de795-7fc4-4850-bc57-525bcb7033f5" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "352de795-7fc4-4850-bc57-525bcb7033f5" ) ) );
 }
 
 
@@ -326,7 +304,7 @@ void DocumentationFigureCreator::createUserLoginDialogFigure()
 		grabDialog( dialog, {}, QStringLiteral("UserLoginDialog.png") );
 	});
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "7310707d-3918-460d-a949-65bd152cb958" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "7310707d-3918-460d-a949-65bd152cb958" ) ) );
 }
 
 
@@ -341,7 +319,7 @@ void DocumentationFigureCreator::createTextMessageDialogFigure()
 		grabDialog( dialog, {}, QStringLiteral("TextMessageDialog.png") );
 	});
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "e75ae9c8-ac17-4d00-8f0d-019348346208" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "e75ae9c8-ac17-4d00-8f0d-019348346208" ) ) );
 }
 
 
@@ -356,83 +334,55 @@ void DocumentationFigureCreator::createOpenWebsiteDialogFigure()
 		grabDialog( dialog, {}, QStringLiteral("OpenWebsiteDialog.png") );
 	});
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "8a11a75d-b3db-48b6-b9cb-f8422ddd5b0c" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "8a11a75d-b3db-48b6-b9cb-f8422ddd5b0c" ) ) );
 }
 
 
 
 void DocumentationFigureCreator::createWebsiteMenuFigure()
 {
-	auto toolbar = m_master->mainWindow()->findChild<MainToolBar *>();
-	auto openWebsiteButton = toolbar->findChild<ToolButton *>( QStringLiteral("OpenWebsite") );
+	const auto openWebsiteButton = m_master.mainWindow()->findChild<ToolButton *>( QStringLiteral("OpenWebsite") );
 
-	auto menu = new QMenu;
-	menu->addAction( QStringLiteral("Intranet") );
-	menu->addAction( QStringLiteral("Wikipedia") );
-	menu->addAction( QIcon( QStringLiteral(":/core/document-edit.png") ), tr("Custom website") );
+	QMenu menu;
+	menu.addAction( QStringLiteral("Intranet") );
+	menu.addAction( QStringLiteral("Wikipedia") );
+	menu.addAction( QIcon( QStringLiteral(":/core/document-edit.png") ), tr("Custom website") );
 
-	scheduleUiOperation( [this, openWebsiteButton, menu]() {
-		scheduleUiOperation( [this, openWebsiteButton, menu]() {
+	openWebsiteButton->setMenu( &menu );
 
-			grabWindow( m_master->mainWindow(), openWebsiteButton->mapTo( m_master->mainWindow(), QPoint( 0, 0 ) ),
-						QSize( qMax( openWebsiteButton->width(), menu->width() ),
-							   openWebsiteButton->height() + menu->height() ),
-						QStringLiteral("OpenWebsiteMenu.png") );
-			menu->close();
-		} );
-
-		menu->close();
-		openWebsiteButton->showMenu();
-	} );
-
-	openWebsiteButton->setMenu( menu );
-	openWebsiteButton->showMenu();
+	grabMenu( m_master.mainWindow(), openWebsiteButton->objectName(), QStringLiteral("OpenWebsiteMenu.png") );
 }
 
 
 
-void DocumentationFigureCreator::createRunProgramDialogFigure()
+void DocumentationFigureCreator::createStartAppDialogFigure()
 {
 	scheduleUiOperation( []() {
 		auto dialog = qobject_cast<QDialog *>( QApplication::activeWindow() );
 		dialog->findChild<QTextEdit *>()->setText( QStringLiteral("notepad") );
 		dialog->setFocus();
 
-		grabDialog( dialog, {}, QStringLiteral("RunProgramDialog.png") );
+		grabDialog( dialog, {}, QStringLiteral("StartAppDialog.png") );
 	});
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "da9ca56a-b2ad-4fff-8f8a-929b2927b442" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "da9ca56a-b2ad-4fff-8f8a-929b2927b442" ) ) );
 }
 
 
 
-void DocumentationFigureCreator::createProgramMenuFigure()
+void DocumentationFigureCreator::createAppMenuFigure()
 {
-	auto toolbar = m_master->mainWindow()->findChild<MainToolBar *>();
-	auto runProgramButton = toolbar->findChild<ToolButton *>( QStringLiteral("RunProgram") );
+	const auto runProgramButton = m_master.mainWindow()->findChild<ToolButton *>( QStringLiteral("StartApp") );
 
-	auto menu = new QMenu;
-	menu->addAction( tr("Open file manager") );
-	menu->addAction( tr("Start learning tool") );
-	menu->addAction( tr("Play tutorial video") );
-	menu->addAction( QIcon( QStringLiteral(":/core/document-edit.png") ), tr("Custom program") );
+	QMenu menu;
+	menu.addAction( tr("Open file manager") );
+	menu.addAction( tr("Start learning tool") );
+	menu.addAction( tr("Play tutorial video") );
+	menu.addAction( QIcon( QStringLiteral(":/core/document-edit.png") ), tr("Custom application") );
 
-	scheduleUiOperation( [this, runProgramButton, menu]() {
-		scheduleUiOperation( [this, runProgramButton, menu]() {
+	runProgramButton->setMenu( &menu );
 
-			grabWindow( m_master->mainWindow(), runProgramButton->mapTo( m_master->mainWindow(), QPoint( 0, 0 ) ),
-						QSize( qMax( runProgramButton->width(), menu->width() ),
-							   runProgramButton->height() + menu->height() ),
-						QStringLiteral("RunProgramMenu.png") );
-			menu->close();
-		} );
-
-		menu->close();
-		runProgramButton->showMenu();
-	} );
-
-	runProgramButton->setMenu( menu );
-	runProgramButton->showMenu();
+	grabMenu( m_master.mainWindow(), runProgramButton->objectName(), QStringLiteral("StartAppMenu.png") );
 }
 
 
@@ -446,7 +396,7 @@ void DocumentationFigureCreator::createRemoteAccessHostDialogFigure()
 		grabDialog( dialog, {}, QStringLiteral("RemoteAccessHostDialog.png") );
 	} );
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "a18e545b-1321-4d4e-ac34-adc421c6e9c8" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "a18e545b-1321-4d4e-ac34-adc421c6e9c8" ) ) );
 }
 
 
@@ -483,25 +433,15 @@ void DocumentationFigureCreator::createRemoteAccessWindowFigure()
 					auto shortcuts = window->findChild<QToolButton *>( QStringLiteral("shortcuts") );
 					Q_ASSERT(shortcuts != nullptr);
 
-					scheduleUiOperation( [this, window, shortcuts]() {
-						auto menu = shortcuts->menu();
-
-						grabWindow( window, shortcuts->mapTo( window, QPoint( 0, 0 ) ),
-									QSize( qMax( shortcuts->width(), menu->width() ),
-										   shortcuts->height() + menu->height() ),
-									QStringLiteral("RemoteAccessShortcutsMenu.png") );
-						menu->close();
-						window->close();
-						m_eventLoop.quit();
-					} );
-
-					shortcuts->showMenu();
+					grabMenu( window, QStringLiteral("shortcuts"), QStringLiteral("RemoteAccessShortcutsMenu.png") );
+					window->close();
+					m_eventLoop.quit();
 				} );
 			} );
 		} );
 	} );
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "ca00ad68-1709-4abe-85e2-48dff6ccf8a2" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "ca00ad68-1709-4abe-85e2-48dff6ccf8a2" ) ) );
 	m_eventLoop.exec();
 }
 
@@ -513,7 +453,7 @@ void DocumentationFigureCreator::createFileTransferDialogFigure()
 
 		auto dialog = qobject_cast<QFileDialog *>( QApplication::activeWindow() );
 		dialog->setDirectory( QDir::current() );
-		dialog->findChildren<QLineEdit *>().first()->setText( QStringLiteral("\"%1.pdf\" \"%2.pdf\"").arg( tr("Handout"), tr("Texts to read") ) );
+		dialog->findChildren<QLineEdit *>().constFirst()->setText( QStringLiteral("\"%1.pdf\" \"%2.pdf\"").arg( tr("Handout"), tr("Texts to read") ) );
 		dialog->setResult( QDialog::Accepted );
 		dialog->setVisible( false );
 
@@ -523,37 +463,68 @@ void DocumentationFigureCreator::createFileTransferDialogFigure()
 			dialog->setFocus();
 			dialog->move( 0, 0 );
 
-			grabWindow( dialog,  QStringLiteral("FileTransferDialogStart.png") );
+			scheduleUiOperation( [dialog, this]() {
 
-			dialog->findChild<QDialogButtonBox *>()->button( QDialogButtonBox::Ok )->click();
+				grabWindow( dialog, QStringLiteral("FileTransferDialogStart.png") );
 
-			scheduleUiOperation( [dialog]() {
-				grabDialog( dialog, {}, QStringLiteral("FileTransferDialogFinished.png") );
-			});
+				dialog->findChild<QDialogButtonBox *>()->button( QDialogButtonBox::Ok )->click();
+
+				scheduleUiOperation( [dialog]() {
+					grabDialog( dialog, {}, QStringLiteral("FileTransferDialogFinished.png") );
+				});
+			} );
 		} );
 	} );
 
-	m_master->runFeature( m_master->featureManager().feature( Feature::Uid( "4a70bd5a-fab2-4a4b-a92a-a1e81d2b75ed" ) ) );
+	m_master.runFeature( VeyonCore::featureManager().feature( Feature::Uid( "4a70bd5a-fab2-4a4b-a92a-a1e81d2b75ed" ) ) );
 }
 
 
 
 void DocumentationFigureCreator::hideComputers()
 {
-	auto view = m_master->mainWindow()->findChild<ComputerMonitoringWidget *>();
+	auto view = m_master.mainWindow()->findChild<ComputerMonitoringWidget *>();
 	view->setSearchFilter( QStringLiteral("XXXXXX") );
 }
 
 
 
-void DocumentationFigureCreator::scheduleUiOperation( const std::function<void(void)>& operation )
+void DocumentationFigureCreator::scheduleUiOperation( const std::function<void(void)>& operation,
+													 QObject* context )
 {
-	QTimer::singleShot( DialogDelay, this, operation );
+	QTimer::singleShot( DialogDelay, context, operation );
 }
 
 
 
-void DocumentationFigureCreator::grabWidget(QWidget* widget, const QPoint& pos, const QSize& size, const QString& fileName)
+void DocumentationFigureCreator::grabMenu( QWidget* window, const QString& buttonName, const QString& fileName )
+{
+	const auto button = window->findChild<ToolButton *>( buttonName );
+
+	scheduleUiOperation( [window, button, &fileName]() {
+		scheduleUiOperation( [window, button, &fileName]() {
+			auto menu = button->menu();
+
+			grabWindow( window, button->mapTo( window, QPoint( 0, 0 ) ),
+						QSize( qMax( button->width(), menu->width() ),
+							   button->height() + menu->height() ),
+						fileName );
+			menu->close();
+		}, window );
+		auto menu = button->menu();
+		menu->close();
+
+		button->setDown(true);
+		button->showMenu();
+	}, window );
+
+	button->setDown(true);
+	button->showMenu();
+}
+
+
+
+void DocumentationFigureCreator::grabWidget(QWidget* widget, QPoint pos, QSize size, const QString& fileName)
 {
 	QPixmap pixmap( size );
 	widget->render( &pixmap, QPoint(), QRegion( QRect( pos, size ) ) );
@@ -562,7 +533,7 @@ void DocumentationFigureCreator::grabWidget(QWidget* widget, const QPoint& pos,
 
 
 
-void DocumentationFigureCreator::grabDialog(QDialog* dialog, const QSize& size, const QString& fileName)
+void DocumentationFigureCreator::grabDialog(QDialog* dialog, QSize size, const QString& fileName)
 {
 	dialog->show();
 	dialog->setFocus();
@@ -593,7 +564,7 @@ void DocumentationFigureCreator::grabWindow(QWidget* widget, const QString& file
 
 
 
-void DocumentationFigureCreator::grabWindow(QWidget* widget, const QPoint& pos, const QSize& size, const QString& fileName)
+void DocumentationFigureCreator::grabWindow(QWidget* widget, QPoint pos, QSize size, const QString& fileName)
 {
 	QGuiApplication::sync();
 
