@@ -1,7 +1,7 @@
 /*
  * MainWindow.cpp - implementation of MainWindow class
  *
- * Copyright (c) 2004-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2004-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -70,6 +70,7 @@ MainWindow::MainWindow( VeyonMaster &masterCore, QWidget* parent ) :
 	ui->statusBar->addWidget( ui->spacerLabel1 );
 	ui->statusBar->addWidget( ui->filterLineEdit, 2 );
 	ui->statusBar->addWidget( ui->filterPoweredOnComputersButton );
+	ui->statusBar->addWidget( ui->filterComputersWithLoggedOnUsersButton );
 	ui->statusBar->addWidget( ui->spacerLabel2, 1 );
 	ui->statusBar->addWidget( ui->gridSizeSlider, 2 );
 	ui->statusBar->addWidget( ui->autoAdjustComputerIconSizeButton );
@@ -128,6 +129,7 @@ MainWindow::MainWindow( VeyonMaster &masterCore, QWidget* parent ) :
 		connect( *it, &QAbstractButton::toggled, it.key(), &QWidget::setVisible );
 	}
 
+	QList<int> splitterSizes;
 	for( auto* splitter : { slideshowSpotlightSplitter, monitoringSplitter, mainSplitter } )
 	{
 		splitter->setHandleWidth( 7 );
@@ -135,31 +137,37 @@ MainWindow::MainWindow( VeyonMaster &masterCore, QWidget* parent ) :
 
 		splitter->installEventFilter( this );
 
-		QList<int> splitterSizes;
 		int index = 0;
 
-		for( const auto& sizeObject : m_master.userConfig().splitterStates()[splitter->objectName()].toArray() )
+		const auto splitterStates = m_master.userConfig().splitterStates()[splitter->objectName()].toArray();
+		splitterSizes.clear();
+		splitterSizes.reserve( splitterStates.size() );
+
+		for( const auto& sizeObject : splitterStates )
 		{
 			auto size = sizeObject.toInt();
 			const auto widget = splitter->widget( index );
 			const auto button = panelButtons.value( widget );
-			if( widget && button )
-			{
-				widget->setVisible( size > 0 );
-				button->setChecked( size > 0 );
-			}
 
-			size = qAbs( size );
-			if( splitter->orientation() == Qt::Horizontal )
+			if( widget )
 			{
-				widget->resize( size, widget->height() );
-			}
-			else
-			{
-				widget->resize( widget->width(), size );
-			}
+				if( button )
+				{
+					widget->setVisible( size > 0 );
+					button->setChecked( size > 0 );
+				}
+				size = qAbs( size );
+				if( splitter->orientation() == Qt::Horizontal )
+				{
+					widget->resize( size, widget->height() );
+				}
+				else
+				{
+					widget->resize( widget->width(), size );
+				}
 
-			widget->setProperty( originalSizePropertyName(), widget->size() );
+				widget->setProperty( originalSizePropertyName(), widget->size() );
+			}
 			splitterSizes.append( size );
 			++index;
 		}
@@ -189,10 +197,13 @@ MainWindow::MainWindow( VeyonMaster &masterCore, QWidget* parent ) :
 
 	// initialize search filter
 	ui->filterPoweredOnComputersButton->setChecked( m_master.userConfig().filterPoweredOnComputers() );
+	ui->filterComputersWithLoggedOnUsersButton->setChecked( m_master.userConfig().filterComputersWithLoggedOnUsers() );
 	connect( ui->filterLineEdit, &QLineEdit::textChanged,
 			 this, [this]( const QString& filter ) { ui->computerMonitoringWidget->setSearchFilter( filter ); } );
 	connect( ui->filterPoweredOnComputersButton, &QToolButton::toggled,
 			 this, [this]( bool enabled ) { ui->computerMonitoringWidget->setFilterPoweredOnComputers( enabled ); } );
+	connect( ui->filterComputersWithLoggedOnUsersButton, &QToolButton::toggled,
+			 this, [this]( bool enabled ) { ui->computerMonitoringWidget->setFilterComputersWithLoggedOnUsers( enabled ); } );
 
 	// initialize monitoring screen size slider
 	ui->gridSizeSlider->setMinimum( ComputerMonitoringWidget::MinimumComputerScreenSize );
@@ -327,7 +338,7 @@ void MainWindow::closeEvent( QCloseEvent* event )
 {
 	if( m_master.currentMode() != VeyonCore::builtinFeatures().monitoringMode().feature().uid() )
 	{
-		const Feature& activeFeature = m_master.featureManager().feature( m_master.currentMode() );
+		const Feature& activeFeature = VeyonCore::featureManager().feature( m_master.currentMode() );
 
 		QMessageBox::information( this, tr( "Feature active" ),
 								  tr( "The feature \"%1\" is still active. Please stop it before closing %2." ).
@@ -337,12 +348,14 @@ void MainWindow::closeEvent( QCloseEvent* event )
 	}
 
 	QJsonObject splitterStates;
-	for( const auto* splitter : findChildren<QSplitter *>() )
+	const auto splitters = findChildren<QSplitter *>();
+	for( const auto* splitter : splitters )
 	{
+		const auto sizes = splitter->sizes();
 		QJsonArray splitterSizes;
 		int i = 0;
 		int hiddenSize = 0;
-		for( auto size : splitter->sizes() )
+		for( auto size : sizes )
 		{
 			auto widget = splitter->widget(i);
 			const auto originalSize = widget->property( originalSizePropertyName() ).toSize();
@@ -422,12 +435,12 @@ void MainWindow::addFeaturesToToolBar()
 {
 	for( const auto& feature : m_master.features() )
 	{
-		if( feature.testFlag( Feature::Internal ) )
+		if( feature.testFlag( Feature::Flag::Meta ) )
 		{
 			continue;
 		}
 
-		ToolButton* btn = new ToolButton( QIcon( feature.iconUrl() ),
+		auto btn = new ToolButton( QIcon( feature.iconUrl() ),
 										  feature.displayName(),
 										  feature.displayNameActive(),
 										  feature.description(),
@@ -435,7 +448,7 @@ void MainWindow::addFeaturesToToolBar()
 		connect( btn, &QToolButton::clicked, this, [=] () {
 			m_master.runFeature( feature );
 			updateModeButtonGroup();
-			if( feature.testFlag( Feature::Mode ) )
+			if( feature.testFlag( Feature::Flag::Mode ) )
 			{
 				reloadSubFeatures();
 			}
@@ -443,7 +456,7 @@ void MainWindow::addFeaturesToToolBar()
 		btn->setObjectName( feature.name() );
 		btn->addTo( ui->toolBar );
 
-		if( feature.testFlag( Feature::Mode ) )
+		if( feature.testFlag( Feature::Flag::Mode ) )
 		{
 			btn->setCheckable( true );
 			m_modeGroup->addButton( btn, buttonId( feature ) );
@@ -462,7 +475,7 @@ void MainWindow::addSubFeaturesToToolButton( QToolButton* button, const Feature&
 		button->setMenu( nullptr );
 	}
 
-	const auto parentFeatureIsMode = parentFeature.testFlag( Feature::Mode );
+	const auto parentFeatureIsMode = parentFeature.testFlag( Feature::Flag::Mode );
 	const auto subFeatures = m_master.subFeatures( parentFeature.uid() );
 
 	if( subFeatures.isEmpty() ||
@@ -497,7 +510,7 @@ void MainWindow::addSubFeaturesToToolButton( QToolButton* button, const Feature&
 				m_master.runFeature( subFeature );
 				if( parentFeatureIsMode )
 				{
-					if( subFeature.testFlag( Feature::Option ) == false )
+					if( subFeature.testFlag( Feature::Flag::Option ) == false )
 					{
 						button->setChecked( true );
 					}
@@ -509,10 +522,10 @@ void MainWindow::addSubFeaturesToToolButton( QToolButton* button, const Feature&
 		action->setToolTip( subFeature.description() );
 		action->setObjectName( subFeature.uid().toString() );
 
-		if( subFeature.testFlag( Feature::Option ) )
+		if( subFeature.testFlag( Feature::Flag::Option ) )
 		{
 			action->setCheckable( true );
-			action->setChecked( subFeature.testFlag( Feature::Checked ) );
+			action->setChecked( subFeature.testFlag( Feature::Flag::Checked ) );
 		}
 	}
 
